import copy
from collections import namedtuple
from itertools import chain
from urllib.parse import parse_qs, urlencode
import json
import math
import pandas as pd

DEFAULT_PAGE = 1
DEFAULT_LIMIT = 50

"""
See pandas docs for to_html params
https://pandas.pydata.org/pandas-docs/version/0.23.4/generated/pandas.DataFrame.to_html.html

See also Extending pandas:
https://pandas.pydata.org/pandas-docs/stable/development/extending.html
"""

Navigation = namedtuple('Navigation',
    ['query', 'limit', 'offset', 'page'])

@pd.api.extensions.register_dataframe_accessor('frameup')
class Frameup:

    def __init__(self, pandas_obj):
        self.default_page = DEFAULT_PAGE
        self.default_limit = DEFAULT_LIMIT
        self._obj = pandas_obj

    passthrough_params = {
        'bold_rows': {
            'type': bool
        },
        'classes': {
            'type': None # special case, list or string
        },
        'escape': {
            'type': bool
        },
        'max_rows': {
            'type': int
        },
        'max_cols': {
            'type': int
        },
        'decimal': {
            'type': str
        },
        'border': {
            'type': int
        },
        'table_id': {
            'type': str
        },
        # buf: not supported
        'columns': {
            'type': list
        },
        'col_space': {
            'type': int
        },
        'header': {
            'type': bool
        },
        'index': {
            'type': bool
        },
        'na_rep': {
            'type': str
        },
        # formatters: not supported
        # 'float_format': not supported
        'sparsify': {
            'type': bool
        },
        'index_names': {
            'type': bool
        },
        'line_width': {
            'type': int
        },
        'justify': {
            'type': str
        }
    }
    int_params = [ k for k,v in passthrough_params.items() if v['type'] == int ]
    bool_params = [ k for k,v in passthrough_params.items() if v['type'] == bool ]

    def prep_params(self, **params):
        """Return subset of query parameters as proper types for passing to
        the to_html of the DataFrame.
        """
        r = {}
        for k,v in params.items():
            if k == 'classes':
                # classes is a special case: comma-separated and/or a list
                r['classes'] = list(chain(*[c.split(',') for c in v]))
                continue
            if isinstance(v, list):
                v = v[0]
            if k in self.int_params:
                r[k] = int(v)
            elif k in self.bool_params:
                if isinstance(v, str):
                    r[k] = v.lower() in ['1', 't', 'true']
            elif k in self.passthrough_params:
                # only string param types should be remaining
                assert self.passthrough_params[k]['type'] == str, \
                    f'{k} is not a string parameter'
                r[k] = v
        return r

    def extract_navigation(self, **params):
        return Navigation(
            query=params.get('query', [''])[0],
            limit=int(params.get('limit', [self.default_limit])[0]),
            offset=int(params.get('offset', [0])[0]),
            page=int(params.get('page', [self.default_page])[0])
        )

    def page_qs(self, path, nav, page):
        nav = nav._replace(page=page)
        return '%s?%s' % (path, urlencode(nav._asdict()))

    def data(self, qs='', path='/', render_html=True, include_data=True, **params):
        if qs:
            params.update(parse_qs(qs))
        results = self._obj
        nav = self.extract_navigation(**params)
        if nav.query:
            results = results.query(nav.query)
        rows = results.shape[0]
        start = (nav.page - 1) * nav.limit + nav.offset
        end = start + nav.limit
        results = results.iloc[ start:end, : ]
        pages = math.ceil(rows / nav.limit)
        prev_page = nav.page - 1 if nav.page > 1 else 1
        next_page = nav.page + 1 if nav.page < pages else pages
        return {
            'rows': rows,
            'page': nav.page,
            'prev_page': prev_page,
            'next_page': next_page,
            'prev': self.page_qs(path, nav, prev_page),
            'next': self.page_qs(path, nav, next_page),
            'first': self.page_qs(path, nav, 1),
            'last': self.page_qs(path, nav, pages),
            'pages': pages,
            'offset': nav.offset,
            'limit': nav.limit,
            'html': results.to_html(**self.prep_params(**params)) if render_html else None,
            'data': results.to_dict() if include_data else None,
            'params': params,
            'query': nav.query
        }

    def json(self, **params):
        return json.dumps(self.data, **params)



import json
from http import HTTPStatus
from wsgiref.simple_server import make_server
import sys
import pandas as pd
from . import frameup

try:
    from jinja2 import Template
    templ = 'frameup/templates/example.j2.html'
    print(f'Using Jinja2 template: {templ}')
    template = Template(open(templ).read())
    render = lambda **kw: template.render(**kw)
except ModuleNotFoundError:
    templ = 'frameup/templates/example.html'
    print(f'Using python template: {templ}')
    template = open(templ).read()
    render = lambda **kw: template.format(**kw)

HOST = 'localhost'
PORT = 8000

STATUS_OK = f'{HTTPStatus.OK} {HTTPStatus.OK.phrase}'


class Server(object):

    def serve(self, environ, start_response):
        data = self.df.frameup.data(
            environ.get('QUERY_STRING', ''),
            justify='center',
            index_names=False,
            classes=['pure-table-striped']
        )
        if 'text/html' in environ['HTTP_ACCEPT']:
            content_type = 'text/html; charset=utf-8'
            response = render(**data).encode('utf8')
        else:
            content_type = 'application/json; charset=utf-8'
            response = json.dumps(data).encode('utf8')
        headers = [
            ('Content-type', content_type),
            ('Content-Length', str(len(response)))
        ]
        start_response(STATUS_OK, headers)
        return [response]


def main():
    """Serve a csv file as a dataframe.
    """
    fn = sys.argv[1]
    server = Server()
    server.df = pd.read_csv(fn)
    server.df.frameup.default_page_size = 5
    with make_server(HOST, PORT, server.serve) as httpd:
        httpd.serve_forever()


if __name__=='__main__':
    main()



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['frameup']

package_data = \
{'': ['*'], 'frameup': ['templates/*']}

install_requires = \
['pandas>=0.23.0,<0.24.0']

entry_points = \
{'console_scripts': ['frameup = frameup.serve:main']}

setup_kwargs = {
    'name': 'frameup',
    'version': '0.1.0',
    'description': 'DataFrames all up in your web applications',
    'long_description': "# DataFrameup\n\nFrameup is the easiest way to get your Pandas DataFrame up into a Python-based web application. Simply `import frameup` and your DataFrames will become URL query parameter, and pagination aware.\n\nZero dependencies, except Pandas of course.\n\n## Quick look\n\nServe a csv as a frameup dataframe on localhost\n\n```\n $ python -m frameup.serve <path-to-csv-file>\n```\n\nThen navigate to http://localhost:8000/. Use the [Pandas DataFrame query syntax](https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.query.html) in the query box.\n\n... or, get a JSON payload:\n\n```\n $ curl 'http://localhost:8000/?query=&limit=10&page=1' | python -m json.tool\n```\n\n\n## Use it in your web application\n\n### Flask example\n\nGiven a template similar to example.js.html\n\n```\nfrom flask import Flask, jsonify, render_template, request, url_for\nimport pandas as pd\nimport frameup\n\napp = Flask(__name__)\n\ndf = pd.read_csv(YOUR_CSV_FILE)\n\n@app.route('/mydataframe')\ndef main():\n    data = df.frameup.data(path=url_for('mydataframe'), **request.args)\n    return render_template('example.j2.html', **data)\n```\n\nFor something ajaxy, just replace the return with:\n\n```\nreturn jsonify(**data)\n```\n\n## Other projects\n\nProjects to review / learn from / use instead\n\n * [Datasette](https://github.com/simonw/datasette)\n * [Workbench](https://workbenchdata.com/)\n",
    'author': 'Scott Bradley',
    'author_email': 'scott@codeslick.com',
    'url': 'https://github.com/scott2b/DataFrameup',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
